@@ -44,7 +44,7 @@ residuals_overall_e_exp <- function(object) { ### experimental non-exported func
   if (obj.eff == "twoways") {
     res_ov_e <- res_ov - ranef(object, "individual")[index(object, "id")] - ranef(object, "time")[index(object, "time")]
   } else {
-    res_ov_e <- res_ov - ranef(object)[index(object, ifelse(obj.eff == "individual", "id", "time"))]
+    res_ov_e <- res_ov - ranef(object)[index(object, if(obj.eff == "individual") "id" else "time")]
   }
   names(res_ov_e) <- names(res_ov)
   return(res_ov_e)
@@ -62,48 +62,11 @@ fitted_exp.plm <- function(x, ...) { #### experimental, non-exported function
   # number of original observations. Thus, model.frame cannot be used but rather
   # pmodel.response because it has the right length. However, pmodel.response
   # shall not be used for the other models because we want the untransformed data.
-  if (model %in% c("between", "fd")) {
-    y <- pmodel.response(x)
-  } else {
-    y <- model.frame(x)[ , 1L]
-  }
+  y <- if (model %in% c("between", "fd")) pmodel.response(x) else model.frame(x)[ , 1L]
   return(y - res)
 }
 
-is.index <- function(index) {
-    # not exported, helper function
-    # checks if the index is an index in the sense of package plm
-    res <- if (all(class(index) == c("pindex", "data.frame"))) TRUE else FALSE
-    return(res)
-}
 
-has.index <- function(object) {
-  # not exported, helper function
-  # checks if an object has an index in sense of package plm
-  # (esp. to distinguish from zoo::index() which always returns an index)
-  index <- attr(object, "index")
-  return(is.index(index))
-}
-
-# pos.index:
-# not exported, helper function
-#
-# determines column numbers of the index variables in a pdata.frame
-# returns named numeric of length 2 or 3 with column numbers of the index variables
-# (1: individual index, 2: time index, if available 3: group index), 
-# names are the names of the index variables
-#
-# returns c(NA, NA) / c(NA, NA, NA) if the index variables are not a column in the pdata.frame
-# (e.g., for pdata.frames created with drop.index = TRUE).
-# Cannot detect index variables if their columns names were changed after creation of the pdata.frame
-
-pos.index <- function(x, ...) {
-  index <- attr(x, "index")
-  index_names <- names(index)
-  index_pos <- match(index_names, names(x))
-  names(index_pos) <- index_names
-  return(index_pos)
-}
 
 # check_propagation_correct_class: helper function
 # Function checks if the class and storage mode (type) of an object match 
