@@ -1,8 +1,3 @@
-data.name <- function(x){
-  data.name <- paste(deparse(x$call$formula))
-  if (length(data.name) > 1) paste(data.name[1], "...")
-  else data.name
-}
 
 #' Hausman Test for Panel Models
 #' 
@@ -13,13 +8,16 @@ data.name <- function(x){
 #' different models.  The `panelmodel` method computes the original
 #' version of the test based on a quadratic form
 #' \insertCite{HAUS:78}{plm}. The `formula` method, if
-#' `method="chisq"` (default), computes the original version of the
-#' test based on a quadratic form; if `method="aux"` then the
-#' auxiliary-regression-based version in Wooldridge (2010,
-#' Sec. 10.7.3.) is computed instead \insertCite{@WOOL:10 Sec.10.7.3}{plm}.
-#' Only the latter can be robustified by specifying a robust
+#' `method = "chisq"` (default), computes the original version of the
+#' test based on a quadratic form; if `method ="aux"` then the
+#' auxiliary-regression-based version as in \insertCite{WOOL:10;textual}{plm},
+#' Sec.10.7.3. Only the latter can be robustified by specifying a robust
 #' covariance estimator as a function through the argument `vcov` (see
 #' **Examples**).
+#'
+#' The `effect` argument is only relevant for the formula method/interface and 
+#' is then applied to both models. For the panelmodel method/interface, the test 
+#' is run with the effects of the already estimated models.
 #' 
 #' The equivalent tests in the **one-way** case using a between
 #' model (either "within vs. between" or "random vs. between")
@@ -33,17 +31,16 @@ data.name <- function(x){
 #' 
 #' @aliases phtest
 #' @param x an object of class `"panelmodel"` or `"formula"`,
-#' @param x2 an object of class `"panelmodel"`,
+#' @param x2 an object of class `"panelmodel"` (only for panelmodel method/interface),
 #' @param model a character vector containing the names of two models
 #' (length(model) must be 2),
+#' @param effect a character specifying the effect to be introduced to both models,
+#'  one of `"individual"`, `"time"`, or `"twoways"` (only for formula method),
 #' @param data a `data.frame`,
 #' @param method one of `"chisq"` or `"aux"`,
 #' @param index an optional vector of index variables,
 #' @param vcov an optional covariance function,
-#' @param \dots further arguments to be passed on. For the formula method,
-#' place argument `effect` here to compare e.g. twoway models
-#' (`effect = "twoways"`) Note: Argument `effect` is not respected in
-#' the panelmodel method.
+#' @param \dots further arguments to be passed on (currently none).
 #' @return An object of class `"htest"`.
 #' @export
 #' @author Yves Croissant, Giovanni Millo
@@ -68,13 +65,14 @@ data.name <- function(x){
 #' re <- plm(form, data = Gasoline, model = "random")
 #' phtest(wi, re)
 #' phtest(form, data = Gasoline)
+#' phtest(form, data = Gasoline, effect = "time")
+#' 
+#' # Regression-based Hausman test
 #' phtest(form, data = Gasoline, method = "aux")
 #' 
-#' # robust Hausman test (regression-based)
+#' # robust Hausman test with vcov supplied as a function and 
+#' # with additional parameters
 #' phtest(form, data = Gasoline, method = "aux", vcov = vcovHC)
-#' 
-#' # robust Hausman test with vcov supplied as a
-#' # function and additional parameters
 #' phtest(form, data = Gasoline, method = "aux",
 #'   vcov = function(x) vcovHC(x, method="white2", type="HC3"))
 #' 
@@ -85,35 +83,36 @@ phtest <- function(x,...){
 #' @rdname phtest
 #' @export
 phtest.formula <- function(x, data, model = c("within", "random"),
+                            effect = c("individual", "time", "twoways"),
                             method = c("chisq", "aux"),
                             index = NULL, vcov = NULL, ...) {
-  # NB: No argument 'effect' here, maybe introduce?
-  #     it gets evaluated due to the eval() call for method="chisq"
-  #     and since rev. 305 due to extraction from dots (...) in method="aux" as a quick fix
-  #    If introduced as argument, change doc accordingly (currently, effect arg is mentioned in ...)
   
-    if (length(model)!=2) stop("two models should be indicated")
+    if (length(model) != 2) stop("two models should be indicated in argument 'model'")
     for (i in 1:2){
         model.name <- model[i]
         if(!(model.name %in% names(model.plm.list))){
             stop("model must be one of ", oneof(model.plm.list))
         }
     }
+  
+    effect <- match.arg(effect)
+  
     switch(match.arg(method),
-           chisq={
+           "chisq" = {
                cl <- match.call(expand.dots = TRUE)
-               cl$model <- model[1]
-               names(cl)[2] <- "formula"
-               m <- match(plm.arg, names(cl), 0)
-               cl <- cl[c(1,m)]
-               cl[[1]] <- as.name("plm")
+               cl$model <- model[1L]
+               cl$effect <- effect
+               names(cl)[2L] <- "formula"
+               m <- match(plm.arg, names(cl), 0L)
+               cl <- cl[c(1L, m)]
+               cl[[1L]] <- as.name("plm")
                plm.model.1 <- eval(cl, parent.frame())
-               plm.model.2 <- update(plm.model.1, model = model[2])
-               return(phtest(plm.model.1, plm.model.2))
+               plm.model.2 <- update(plm.model.1, model = model[2L])
+               return(phtest(plm.model.1, plm.model.2)) # exit to phtest.panelmodel
            },
-           aux={
+           "aux" = {
                ## some interface checks here
-               if (model[1] != "within") {
+               if (model[1L] != "within") {
                    stop("Please supply 'within' as first model type")
                }
              
@@ -122,20 +121,14 @@ phtest.formula <- function(x, data, model = c("within", "random"),
                ## set pdata.frame
                if (!inherits(data, "pdata.frame")) data <- pdata.frame(data, index = index) #, ...)
 
-               row.names(data) <- NULL # reset rownames of original data set (->numbers rownames in clean sequence) to make rownames
-                                       # comparable for later comparison to obs used in estimation of models (get rid of NA values)
-                                       # [needed because pmodel.response() and model.matrix() do not retain fancy rownames, but rownames]
+               # reset rownames of original data set (->numbers rownames in clean sequence) to make rownames
+               # comparable for later comparison to obs used in estimation of models (get rid of NA values)
+               # [needed because pmodel.response() and model.matrix() do not retain fancy rownames, but rownames]
+               row.names(data) <- NULL
                
-               # rev. 305: quick and dirty fix for missing effect argument in function 
-               # signature for formula interface/test="aux": see if effect is in dots and extract
-                  dots <- list(...)
-                  # print(dots) # DEBUG printing
-               if (!is.null(dots$effect)) effect <- dots$effect else effect <- NULL
-               # calculatate FE and RE model
-
-               fe_mod <- plm(formula = x, data = data, model = model[1], effect = effect)
-
-               re_mod <- plm(formula = x, data = data, model = model[2], effect = effect)
+               # calculate FE and RE model
+               fe_mod <- plm(formula = x, data = data, model = model[1L], effect = effect)
+               re_mod <- plm(formula = x, data = data, model = model[2L], effect = effect)
 
                 ## DEBUG printing:
                  # print(effect)
@@ -144,56 +137,58 @@ phtest.formula <- function(x, data, model = c("within", "random"),
                  # print(paste0("mod2: ", describe(re_mod, "effect")))
                  # print(fe_mod)
                  # print(re_mod)
+               
                reY <- pmodel.response(re_mod)
 #               reX <- model.matrix(re_mod)[ , -1, drop = FALSE] # intercept not needed; drop=F needed to prevent matrix
-#               feX <- model.matrix(fe_mod, cstcovar.rm = TRUE)                      # from degenerating to vector if only one regressor
+#               feX <- model.matrix(fe_mod, cstcovar.rm = TRUE)  # from degenerating to vector if only one regressor
                reX <- model.matrix(re_mod, cstcovar.rm = "intercept")
                feX <- model.matrix(fe_mod, cstcovar.rm = "all")
 
-               dimnames(feX)[[2]] <- paste(dimnames(feX)[[2]], "tilde", sep=".")
                ## estimated models could have fewer obs (due dropping of NAs) compared to the original data
                ## => match original data and observations used in estimated models
                ## routine adapted from lmtest::bptest
                commonrownames <- intersect(intersect(intersect(row.names(data), names(reY)), row.names(reX)), row.names(feX))
-               if (!(all(c(row.names(data) %in% commonrownames, commonrownames %in% row.names(data))))) {
-                 data <- data[commonrownames, ]
-                 reY  <- reY[commonrownames]
-                 reX  <- reX[commonrownames, ]
-                 feX  <- feX[commonrownames, ]
+               if(!(all(c(row.names(data) %in% commonrownames, commonrownames %in% row.names(data))))) {
+                 data <- data[commonrownames, , drop = FALSE]
+                 reY <- reY[commonrownames]
+                 reX <- reX[commonrownames, , drop = FALSE]
+                 feX <- feX[commonrownames, , drop = FALSE]
                }
                
                # Tests of correct matching of obs (just for safety ...)
-               if (!all.equal(length(reY), nrow(data), nrow(reX), nrow(feX)))
+               if(!all(c(isTRUE(all.equal(length(reY), nrow(data))), isTRUE(all.equal(nrow(reX), nrow(feX))), 
+                         isTRUE(all.equal(length(reY), nrow(reX))))))
                   stop("number of cases/observations do not match, most likely due to NAs in \"data\"")
-                if (any(c(is.na(names(reY)), is.na(row.names(data)), is.na(row.names(reX)), is.na(row.names(feX)))))
+                if(any(c(is.na(names(reY)), is.na(row.names(data)), is.na(row.names(reX)), is.na(row.names(feX)))))
                     stop("one (or more) rowname(s) is (are) NA")
-                if (!all.equal(names(reY), row.names(data), row.names(reX), row.names(feX)))
+                if(!all(c(isTRUE(all.equal(names(reY), row.names(data))), isTRUE(all.equal(row.names(reX), row.names(feX))),
+                          isTRUE(all.equal(names(reY), row.names(reX))))))
                   stop("row.names of cases/observations do not match, most likely due to NAs in \"data\"")
 
-               ## fetch indices here, check pdata
                ## construct data set and formula for auxiliary regression
-               data <- pdata.frame(cbind(index(data), reY, reX, feX))
+               dimnames(feX)[[2L]] <- paste(dimnames(feX)[[2L]], "tilde", sep=".")
+               auxdata <- pdata.frame(cbind(index(data), reY, reX, feX))
                auxfm <- as.formula(paste("reY~",
-                                         paste(dimnames(reX)[[2]],
-                                               collapse="+"), "+",
-                                         paste(dimnames(feX)[[2]],
-                                               collapse="+"), sep=""))
-               auxmod <- plm(formula = auxfm, data = data, model = "pooling")
-               nvars <- dim(feX)[[2]]
+                                         paste(dimnames(reX)[[2L]], collapse="+"),
+                                         "+",
+                                         paste(dimnames(feX)[[2L]], collapse="+"), sep=""))
+               auxmod <- plm(formula = auxfm, data = auxdata, model = "pooling")
+               nvars <- dim(feX)[[2L]]
                R <- diag(1, nvars)
                r <- rep(0, nvars) # here just for clarity of illustration
-               omega0 <- vcov(auxmod)[(nvars+2):(nvars*2+1),
-                                      (nvars+2):(nvars*2+1)]
-               Rbr <- R %*% coef(auxmod)[(nvars+2):(nvars*2+1)] - r
-
-               h2t <- as.numeric(crossprod(Rbr, solve(omega0, Rbr)))
-               ph2t <- pchisq(h2t, df = nvars, lower.tail = FALSE)
-
+               range <- (nvars+2L):(nvars*2L + 1L)
+               omega0 <- vcov(auxmod)[range, range]
+               Rbr <- crossprod(R, coef(auxmod)[range]) - r
+               
                df <- nvars
+               
+               h2t <- as.numeric(crossprod(Rbr, solve(omega0, Rbr)))
+               ph2t <- pchisq(h2t, df = df, lower.tail = FALSE)
+
                names(df) <- "df"
                names(h2t) <- "chisq"
 
-               if (!is.null(vcov)) {
+               if(!is.null(vcov)) {
                    vcov <- paste(", vcov: ",
                                   paste(deparse(substitute(vcov))),
                                   sep="")
@@ -205,7 +200,8 @@ phtest.formula <- function(x, data, model = c("within", "random"),
                              method      = paste("Regression-based Hausman test",
                                                   vcov, sep=""),
                              alternative = "one model is inconsistent",
-                             data.name   = paste(deparse(substitute(x))))
+                             data.name   = paste(deparse(substitute(x))),
+                             auxmod      = auxmod)
                class(haus2) <- "htest"
                return(haus2)
            })
@@ -213,7 +209,7 @@ phtest.formula <- function(x, data, model = c("within", "random"),
 
 #' @rdname phtest
 #' @export
-phtest.panelmodel <- function(x, x2, ...){
+phtest.panelmodel <- function(x, x2, ...) {
   coef.wi <- coef(x)
   coef.re <- coef(x2)
   vcov.wi <- vcov(x)
@@ -221,8 +217,8 @@ phtest.panelmodel <- function(x, x2, ...){
   names.wi <- names(coef.wi)
   names.re <- names(coef.re)
   common_coef_names <- names.re[names.re %in% names.wi]
-  coef.h <- common_coef_names[!(common_coef_names %in% "(Intercept)")] # drop intercept if included (relevant when between model inputed)
-  if(length(coef.h) == 0) stop("no common coefficients in models")
+  coef.h <- common_coef_names[!(common_coef_names %in% "(Intercept)")] # drop intercept if included (relevant when between model input)
+  if(length(coef.h) == 0L) stop("no common coefficients in models")
   dbeta <- coef.wi[coef.h] - coef.re[coef.h]
   df <- length(dbeta)
   dvcov <- vcov.wi[coef.h, coef.h] - vcov.re[coef.h, coef.h]
@@ -237,13 +233,13 @@ phtest.panelmodel <- function(x, x2, ...){
     # Tests with between model do not extend to two-ways case -> give error
     # There are, however, some equiv. tests with the individual/time between 
     # model, but let's not support them (see Kang (1985), Baltagi (2013), Sec. 4.3.7)
-    if (   (modx  == "between" | modx2 == "between")
-        && (effx == "twoways" | effx2 == "twoways")) stop("tests with between model in twoways case not supported")
+    if (   (modx  == "between" || modx2 == "between")
+        && (effx == "twoways" || effx2 == "twoways")) stop("tests with between model in twoways case not supported")
     
     # in case of one-way within vs. between (m3 in Baltagi (2013), pp. 77, 81)
     # the variances need to be added (not subtracted like in the other cases)
-    if (  (modx  == "within" && modx2 == "between")
-        | (modx2 == "within" && modx  == "between")) {
+    if (   (modx  == "within" && modx2 == "between")
+        || (modx2 == "within" && modx  == "between")) {
       dvcov <- vcov.wi[coef.h, coef.h] + vcov.re[coef.h, coef.h]
     }
   #### END cater for equivalent tests with between model
@@ -254,7 +250,6 @@ phtest.panelmodel <- function(x, x2, ...){
   parameter <- df
   names(parameter) <- "df"
   alternative <- "one model is inconsistent"
-#  null.value <- "both models are consistent"
   
   ## DEBUG printing:
      # print(paste0("mod1: ", describe(x,  "effect")))
@@ -265,7 +260,6 @@ phtest.panelmodel <- function(x, x2, ...){
               parameter    = parameter,
               method       = "Hausman Test",
               data.name    = data.name(x),
- #             null.value  = null.value,
               alternative  = alternative)
   class(res) <- "htest"
   return(res)
@@ -294,8 +288,7 @@ phtest.panelmodel <- function(x, x2, ...){
 # DOI: 10.1002/0471667196.ess0265.pub2
 pchibarsq <- function(q, df, weights, lower.tail = TRUE, ... ) {
   # NB: other parameters in dots (...): not checked if valid! (ncp, log, ...)
-  res <- sum(weights * pchisq(q, df = df, lower.tail = lower.tail, ...))
-  return(res)
+  sum(weights * pchisq(q, df = df, lower.tail = lower.tail, ...))
 }
 
 
@@ -308,7 +301,7 @@ pchibarsq <- function(q, df, weights, lower.tail = TRUE, ... ) {
 #' These Lagrange multiplier tests use only the residuals of the
 #' pooling model.  The first argument of this function may be either a
 #' pooling model of class `plm` or an object of class `formula`
-#' describing the model. For inputted within (fixed effects) or random
+#' describing the model. For input within (fixed effects) or random
 #' effects models, the corresponding pooling model is calculated
 #' internally first as the tests are based on the residuals of the
 #' pooling model.
@@ -402,21 +395,20 @@ plmtest.plm <- function(x,
   type <- match.arg(type)
   if (describe(x, "model") != "pooling") x <- update(x, model = "pooling")
   pdim <- pdim(x)
-  n <- pdim$nT$n
-  T <- pdim$nT$T
   N_obs <- pdim$nT$N
   balanced <- pdim$balanced
-  index <- attr(model.frame(x), "index")
-  id <- index[[1]]
-  time <- index[[2]]
+  index <- unclass(attr(model.frame(x), "index")) # unclass for speed
+  id <- index[[1L]]
+  time <- index[[2L]]
   T_i <- pdim$Tint$Ti
   N_t <- pdim$Tint$nt
   res <- resid(x)
   
   ### calc of parts of test statistic ##
-  # calc. is done w/o using matrix calculation, see e.g. Baltagi/Li (1990), p. 106
-  A1 <- as.numeric(crossprod(tapply(res, id, sum)) / sum(res ^ 2) - 1)   # == A1 <- sum(tapply(res,id,sum)^2)/sum(res^2) - 1
-  A2 <- as.numeric(crossprod(tapply(res, time, sum)) / sum(res ^ 2) - 1) # == A2 <- sum(tapply(res,time,sum)^2)/sum(res^2) - 1
+  # calc. is done w/o using matrix calculation, see, e.g., Baltagi/Li (1990), p. 106
+  CP.res <- crossprod(res)
+  A1 <- as.numeric(crossprod(tapply(res, id,   sum)) / CP.res - 1) # == A1 <- sum(tapply(res,id,sum)^2)   / sum(res^2) - 1
+  A2 <- as.numeric(crossprod(tapply(res, time, sum)) / CP.res - 1) # == A2 <- sum(tapply(res,time,sum)^2) / sum(res^2) - 1
   
   M11 <- sum(T_i ^ 2)
   M22 <- sum(N_t ^ 2)
@@ -431,7 +423,7 @@ plmtest.plm <- function(x,
     if (!type %in% c("honda", "bp", "kw"))
       stop("type must be one of \"honda\", \"bp\" or \"kw\" for a one way model") # kw oneway coincides with honda
     
-    stat <- ifelse(effect == "individual", LM1, LM2)
+    stat <- if(effect == "individual") LM1 else LM2
     stat <- switch(type,
                      honda = c(normal = stat),
                      bp    = c(chisq  = stat ^ 2),
@@ -480,10 +472,8 @@ plmtest.plm <- function(x,
                             time    = "time effects",
                             twoways = "two-ways effects")
   
-  balanced.type <- ifelse(balanced, "balanced", "unbalanced")
-  
   method <- paste("Lagrange Multiplier Test - ", method.effect,
-                  " (", method.type, ") for ", balanced.type, " panels", sep="")
+                  " (", method.type, ")", sep="")
   
   if (type %in% c("honda", "kw")) {
     RVAL <- list(statistic = stat,
@@ -516,10 +506,10 @@ plmtest.formula <- function(x, data, ...,
   cl$model <- "pooling" # plmtest is performed on the pooling model...
   cl$effect <- NULL     # ... and pooling model has no argument effect...
   cl$type <- NULL       # ... and no argument type => see below: pass on args effect and type to plmtest.plm()
-  names(cl)[2] <- "formula"
-  m <- match(plm.arg, names(cl), 0)
-  cl <- cl[c(1,m)]
-  cl[[1]] <- as.name("plm")
+  names(cl)[2L] <- "formula"
+  m <- match(plm.arg, names(cl), 0L)
+  cl <- cl[c(1L, m)]
+  cl[[1L]] <- as.name("plm")
   plm.model <- eval(cl, parent.frame())
   plmtest(plm.model, effect = effect, type = type) # pass on args effect and type to plmtest.plm()
 }
@@ -570,10 +560,10 @@ pFtest <- function(x, ...){
 pFtest.formula <- function(x, data, ...){
   cl <- match.call(expand.dots = TRUE)
   cl$model <- "within"
-  names(cl)[2] <- "formula"
-  m <- match(plm.arg,names(cl),0)
-  cl <- cl[c(1,m)]
-  cl[[1]] <- as.name("plm")
+  names(cl)[2L] <- "formula"
+  m <- match(plm.arg,names(cl), 0L)
+  cl <- cl[c(1L, m)]
+  cl[[1L]] <- as.name("plm")
   plm.within <- eval(cl,parent.frame())
   plm.pooling <- update(plm.within, model = "pooling")
   pFtest(plm.within, plm.pooling, ...)
@@ -602,7 +592,7 @@ pFtest.plm <- function(x, z, ...){
   alternative <- "significant effects"
   res <- list(statistic   = stat,
               p.value     = pval,
-              method      = paste("F test for ", effect, " effects",sep=""),
+              method      = paste("F test for ", effect, " effects", sep=""),
               parameter   = parameter,
               data.name   = data.name(x),
               alternative = alternative)
@@ -611,8 +601,9 @@ pFtest.plm <- function(x, z, ...){
 }
 
 ############## pwaldtest() ############################################
-# pwaldtest is used in summary.plm, summary.pht to compute the F statistic, but can be used 
-# as a stand-alone test of joint significance of all slopes
+# pwaldtest is used in summary.plm, summary.pht, summary.pgmm to compute the
+# Chi-square or F statistic, but can be used as a stand-alone test of
+# joint significance of all slopes
 #
 # Short intro (but see associated help file)
 # arg 'vcov' non-NULL => the robust tests are carried out
@@ -622,48 +613,18 @@ pFtest.plm <- function(x, z, ...){
 # Chi-sq test for IV models as in Wooldridge (1990), A note on the Lagrange multiplier and F-statistics for two stage least
 #                                                    squares regressions, Economics Letters 34: 151-155.
 
-# trans_clubSandwich_vcov: helper function for pwaldtest()
-# translate vcov object from package clubSandwich so it is suitable for summary.plm, plm's pwaldtest.
-# Attribute "cluster" in clubSandwich's vcov objects contains the cluster variable itself.
-# plm's vcov object also has attribute "cluster" but it contains a character as
-# information about the cluster dimension (either "group" or "time")
-#
-# inputs:
-#   * CSvcov: a vcov as returned by clubSandwich's vcovCR function [class c("vcovCR", "clubSandwich")]
-#   * index: the index belonging to a plm object/model
-# return value:
-#   * modified CSvcov (substituted attribute "cluster" with suitable character or NULL)
-
-trans_clubSandwich_vcov <- function(CSvcov, index) {
-  clustervar <- attr(CSvcov, "cluster")
-  if (!is.null(clustervar)) {
-      if (isTRUE(all.equal(index[[1]], clustervar))) {
-        attr(CSvcov, "cluster") <- "group"
-        return(CSvcov)
-      }
-      if (isTRUE(all.equal(index[[2]], clustervar))) {
-        attr(CSvcov, "cluster") <- "time"
-        return(CSvcov)
-      } else {
-        attr(CSvcov, "cluster") <- NULL
-        return(CSvcov)
-      }
-  }
-  warning("no attribute \"cluster\" found in supplied vcov object")
-  return(CSvcov)
-}
-
 #' Wald-style Chi-square Test and F Test
 #' 
 #' Wald-style Chi-square test and F test of slope coefficients being
 #' zero jointly, including robust versions of the tests.
 #' 
 #' 
-#' `pwaldtest` can be used stand--alone with a plm object or a pvcm
-#' object (for the latter only the 'random' type is valid and no
-#' further arguments are processed). It is also used in
+#' `pwaldtest` can be used stand--alone with a plm object, a pvcm object,
+#' and a pgmm object (for pvcm objects only the 'random' type is valid and no
+#' further arguments are processed; for pgmm objects only arguments `param`
+#' and `vcov` are valid). It is also used in
 #' [summary.plm()] to produce the F statistic and the Chi-square
-#' statistic for the joint test of coefficients.
+#' statistic for the joint test of coefficients and in [summary.pgmm()].
 #' 
 #' `pwaldtest` performs the test if the slope coefficients of a panel
 #' regression are jointly zero. It does not perform general purpose
@@ -678,8 +639,8 @@ trans_clubSandwich_vcov <- function(CSvcov, index) {
 #' supplying argument (`.df2`)), the adjustment of the second degrees
 #' of freedom parameter is performed by default. The second degrees of
 #' freedom parameter is adjusted to be the number of unique elements
-#' of the cluster variable - 1, e. g. the number of individuals -
-#' 1. For the degrees of freedom adjustment of the F test in general,
+#' of the cluster variable - 1, e. g., the number of individuals minus 1.
+#' For the degrees of freedom adjustment of the F test in general,
 #' see e. g. \insertCite{CAME:MILL:15;textual}{plm}, section VII;
 #' \insertCite{ANDR:GOLS:SCMI:13}{plm}, pp. 126, footnote 4.
 #' 
@@ -697,7 +658,7 @@ trans_clubSandwich_vcov <- function(CSvcov, index) {
 #' 
 #' @aliases pwaldtest
 #' @param x an estimated model of which the coefficients should be
-#'     tested (usually of class `"plm"`/`"pvcm"`),
+#'     tested (usually of class `"plm"`/`"pvcm"`/`"pgmm"`)`,
 #' @param test a character, indicating the test to be performed, may
 #'     be either `"Chisq"` or `"F"` for the Wald-style
 #'     Chi-square test or F test, respectively,
@@ -717,8 +678,12 @@ trans_clubSandwich_vcov <- function(CSvcov, index) {
 #'     used),
 #' @param .df2 a numeric, used if one wants to overwrite the second
 #'     degrees of freedom parameter for the F test (usually not used),
+#' @param param (for pgmm method only): select the parameters to be tested:
+#'     `"coef"`, `"time"`, or `"all"``.
 #' @param \dots further arguments (currently none).
-#' @return An object of class `"htest"`.
+#' @return An object of class `"htest"`, except for pvcm's within model for which
+#'         a data.frame with results of the Wald chi-square tests and F tests per
+#'         regression is returned.
 #' @export
 #' @author Yves Croissant (initial implementation) and Kevin Tappe
 #'     (extensions: vcov argument and F test's df2 adjustment)
@@ -765,19 +730,18 @@ pwaldtest.plm <- function(x, test = c("Chisq", "F"), vcov = NULL,
                           df2adj = (test == "F" && !is.null(vcov) && missing(.df2)), .df1, .df2, ...) {
   model <- describe(x, "model")
   test <- match.arg(test)
-  df1 <- ifelse(model == "within",
-                length(coef(x)),
-                length(coef(x)) - has.intercept(x))
+  df1 <- if(model == "within") length(coef(x)) else { length(coef(x)) - has.intercept(x) }
   df2 <- df.residual(x)
-  tss <- tss(x)
-  ssr <- deviance(x)
+#  tss <- tss(x)        # not good for models without intercept
+#  ssr <- deviance(x)   # -- " --
   vcov_arg <- vcov
   int <- "(Intercept)"
   coefs_wo_int <- coef(x)[!(names(coef(x)) %in% int)]
-  
+  if(!length(coefs_wo_int)) stop(paste("No non-intercept regressors in input model 'x',",
+                                       "cannot perform Wald joint significance test"))
   # sanity check
   if (df2adj == TRUE && (is.null(vcov_arg) || test != "F")) {
-    stop("df2adj == TRUE sensible only for robust F test, i.e. test == \"F\" and !is.null(vcov) and missing(.df2)")
+    stop("df2adj == TRUE sensible only for robust F test, i.e., test == \"F\" and !is.null(vcov) and missing(.df2)")
   }
   
   # if robust test: prepare robust vcov
@@ -792,7 +756,7 @@ pwaldtest.plm <- function(x, test = c("Chisq", "F"), vcov = NULL,
       attr(rvcov, which = "cluster") <- attr(rvcov_orig, which = "cluster") # restore dropped 'cluster' attribute
     }
     # if robust F test: by default, do finite-sample adjustment for df2
-    if (df2adj == TRUE & test == "F") {
+    if (df2adj == TRUE && test == "F") {
       # determine the variable that the clustering is done on by
       # attribute "cluster" in the vcov (matrix object)
       # if only one member in cluster: fall back to original df2
@@ -826,73 +790,50 @@ pwaldtest.plm <- function(x, test = c("Chisq", "F"), vcov = NULL,
   if (!missing(.df2)) df2 <- .df2
   
   if (test == "Chisq"){
-    # perform "normal" chisq test
+    # perform non-robust chisq test
     if (is.null(vcov_arg)) {
-      stat <- if(length(formula(x))[2] > 1) {
-        # IV case: cannot take usual TSS-SSR-way to calc. stat
-        as.numeric(crossprod(solve(vcov(x)[names(coefs_wo_int), names(coefs_wo_int)], coefs_wo_int), coefs_wo_int))
-      } else {
-        # non IV
-        (tss-ssr)/(ssr/df2)
-      }
-      
+      names.coefs_wo_int <- names(coefs_wo_int)
+      stat <- as.numeric(crossprod(solve(vcov(x)[names.coefs_wo_int, names.coefs_wo_int], coefs_wo_int), coefs_wo_int))
+#     stat < - (tss-ssr)/(ssr/df2) # does not produce correct results for unbalanced RE models and (un)balanced IV models
       names(stat) <- "Chisq"
       pval <- pchisq(stat, df = df1, lower.tail = FALSE)
       parameter <- c(df = df1)
-      method <- "Wald test"
+      method <- "Wald test for joint significance"
     } else {
       # perform robust chisq test
-      
-      # alternative:
-      # use package car for statistic:
-      # Note: has.intercept() returns TRUE for FE models, so do not use it here...
-      # return_car_lH <- car::linearHypothesis(x,
-      #                                        names(coef(x))[if ("(Intercept)" %in% names(coef(x))) -1 else TRUE],
-      #                                        test="Chisq",
-      #                                        vcov. = rvcov_orig)
-      # stat_car <- return_car_lH[["Chisq"]][2] # extract statistic
-      
       stat <- as.numeric(crossprod(solve(rvcov, coefs_wo_int), coefs_wo_int))
       names(stat) <- "Chisq"
       pval <- pchisq(stat, df = df1, lower.tail = FALSE)
       parameter <- c(df = df1)
-      method <- paste0("Wald test (robust)", rvcov_name)
+      method <- paste0("Wald test for joint significance (robust)", rvcov_name)
     }
   }
   if (test == "F"){
-    if(length(formula(x))[2] > 1) stop("test = \"F\" not sensible for IV models")
+    if(length(formula(x))[2L] > 1L) stop("test = \"F\" not sensible for IV models")
     if (is.null(vcov_arg)) {
       # perform "normal" F test
-      stat <- (tss-ssr)/ssr*df2/df1
+      names.coefs_wo_int <- names(coefs_wo_int)
+      stat <- as.numeric(crossprod(solve(vcov(x)[names.coefs_wo_int, names.coefs_wo_int], coefs_wo_int), coefs_wo_int)) / df1
+#      stat <- (tss-ssr)/ssr*df2/df1 # does not produce correct results for unbalanced RE models
       names(stat) <- "F"
       pval <- pf(stat, df1 = df1, df2 = df2, lower.tail = FALSE)
       parameter <- c(df1 = df1, df2 = df2)
-      method <- "F test"
+      method <- "F test for joint significance"
     } else {
       # perform robust F test
-      
-      # alternative:
-      # use package car for statistic:
-      # Note: has.intercept() returns TRUE for FE models, so do not use it here...
-      # Note: car::linearHypothesis does not adjust df2 for clustering
-      # return_car_lH <- car::linearHypothesis(x,
-      #                                        names(coef(x))[if ("(Intercept)" %in% names(coef(x))) -1 else TRUE],
-      #                                        test="F",
-      #                                        vcov. = rvcov_orig)
-      # stat_car <- return_car_lH[["F"]][2] # extract statistic
-      
       stat <- as.numeric(crossprod(solve(rvcov, coefs_wo_int), coefs_wo_int) / df1)
       names(stat) <- "F"
       pval <- pf(stat, df1 = df1, df2 = df2, lower.tail = FALSE)
-      parameter <- c(df1 = df1, df2 = df2) # Dfs
-      method  <- paste0("F test (robust)", rvcov_name)
+      parameter <- c(df1 = df1, df2 = df2)
+      method  <- paste0("F test for joint significance (robust)", rvcov_name)
     }
   }
   res <- list(data.name = data.name(x),
               statistic = stat,
               parameter = parameter,
               p.value   = pval,
-              method    = method
+              method    = method,
+              alternative = "at least one coefficient is not null"
   )
   class(res) <- "htest"
   return(res)
@@ -902,30 +843,158 @@ pwaldtest.plm <- function(x, test = c("Chisq", "F"), vcov = NULL,
 #' @export
 pwaldtest.pvcm <- function(x, ...) {
   model <- describe(x, "model")
-  if(!model == "random") stop("pwaldtest.pvcm only applicable to 'random' pvcm objects")
+  effect <- describe(x, "effect")
+
+  coefs.no.int <- !names(x$coefficients) %in% "(Intercept)" # logical with non-intercept regressors set to TRUE
+  if(!length(names(x$coefficients)[coefs.no.int])) {
+    # error informatively if only-intercept model (no other regressors)
+    stop(paste("No non-intercept regressors in model(s) of input 'x',",
+               "cannot perform Wald joint significance test(s)"))
+  }
+
+  if(model == "within") {
+    # for the within case, simply return a data.frame with all test results
+    # of single estimations (per individual or per time period)
+    
+    ii <- switch(effect, "individual" = 1L, "time" = 2L)
+    
+    residl <- collapse::gsplit(x$residuals, unclass(index(x))[[ii]], use.names = FALSE)
+    
+    # vcovs and coefficients w/o intercept
+    vcovl <- lapply(x$vcov, function(x) x[coefs.no.int, coefs.no.int])
+    coefl <- as.list(data.frame(t(x$coefficients[ , coefs.no.int, drop = FALSE])))
+    df1 <- ncol(x$coefficients[ , coefs.no.int, drop = FALSE]) # ncol is same df1 for all models (as all models estimate the same coefs)
+    df2 <- lengths(residl) - ncol(x$coefficients) # (any intercept is subtracted)
+    
+    statChisqs <- mapply(FUN = function(v, c) as.numeric(crossprod(solve(v, c), c)),
+                     vcovl, coefl)
+    statFs <- statChisqs / df1
+    
+    pstatChisqs <- pchisq(statChisqs, df = df1, lower.tail = FALSE)
+    pstatFs <- pf(statFs, df1 = df1, df2 = df2, lower.tail = FALSE)
+    
+    stats.pvcm.within <- as.data.frame(cbind("Chisq"    = statChisqs,
+                                             "p(chisq)" = pstatChisqs,
+                                             "F"        = statFs,
+                                             "p(F)"     = pstatFs,
+                                             "df1"      = rep(df1, length(residl)),
+                                             "df2"      = df2))
+    # early return
+    return(stats.pvcm.within)
+  }
   
-  coefs_wo_int <- x$coefficients[setdiff(names(x$coefficients), "(Intercept)")]
-  stat <- as.numeric(crossprod(solve(vcov(x)[names(coefs_wo_int), names(coefs_wo_int)], coefs_wo_int), coefs_wo_int))
+  ## case: model == "random"
+  coefs_wo_int <- x$coefficients[coefs.no.int]
+  stat <- as.numeric(crossprod(solve(vcov(x)[coefs.no.int, coefs.no.int], coefs_wo_int), coefs_wo_int))
   names(stat) <- "Chisq"
   df1 <- length(coefs_wo_int)
   pval <- pchisq(stat, df = df1, lower.tail = FALSE)
   parameter <- c(df = df1)
-  method <- "Wald test"
+  method <- "Wald test for joint significance"
   
   res <- list(data.name = data.name(x),
               statistic = stat,
               parameter = parameter,
               p.value   = pval,
-              method    = method
+              method    = method,
+              alternative = "at least one coefficient is not null"
   )
   class(res) <- "htest"
   return(res)
 }
 
+
+#' @rdname pwaldtest
+#' @export
+pwaldtest.pgmm <- function(x, param = c("coef", "time", "all"), vcov = NULL, ...) {
+  param <- match.arg(param)
+  vcov_supplied <- !is.null(vcov)
+  myvcov <- vcov
+  if (is.null(vcov)) vv <- vcov(x)
+  else if (is.function(vcov)) vv <- myvcov(x)
+  else vv <- myvcov
+
+  model <- describe(x, "model")
+  effect <- describe(x, "effect")
+  if (param == "time" && effect == "individual") stop("no time dummies in this model")
+  transformation <- describe(x, "transformation")
+  coefficients <- if(model == "onestep") x$coefficients else x$coefficients[[2L]]
+  Ktot <- length(coefficients)
+  Kt <- length(x$args$namest)
+  
+  switch(param,
+         "time" = {
+           start <- Ktot - Kt + if(transformation == "ld") 2 else 1
+           end <- Ktot
+         },
+         "coef" = {
+           start <- 1
+           end <- if (effect == "twoways") Ktot - Kt else Ktot
+         },
+         "all" = {
+           start <- 1
+           end <- Ktot
+         })
+  coef <- coefficients[start:end]
+  vv <- vv[start:end, start:end]
+  stat <- as.numeric(crossprod(coef, crossprod(solve(vv), coef)))
+  names(stat) <- "chisq"
+  parameter <- length(coef)
+  names(parameter) <- "df"
+  pval <- pchisq(stat, df = parameter, lower.tail = FALSE)
+  method <- "Wald test for joint significance"
+  if (vcov_supplied) {
+    rvcov_name <- paste0(", vcov: ", paste0(deparse(substitute(vcov))))
+    method <- paste0(method, " (robust)", rvcov_name)
+  }
+  wald.pgmm <- list(statistic = stat,
+                    p.value   = pval,
+                    parameter = parameter,
+                    method    = method,
+                    alternative = "at least one coefficient is not null",
+                    data.name = data.name(x))
+  class(wald.pgmm) <- "htest"
+  return(wald.pgmm)
+}
+
+#' @export
 pwaldtest.default <- function(x, ...) {
   pwaldtest.plm(x, ...)
 }
 
+
+# trans_clubSandwich_vcov: helper function for pwaldtest()
+# translate vcov object from package clubSandwich so it is suitable for summary.plm, plm's pwaldtest.
+# Attribute "cluster" in clubSandwich's vcov objects contains the cluster variable itself.
+# plm's vcov object also has attribute "cluster" but it contains a character as
+# information about the cluster dimension (either "group" or "time")
+#
+# inputs:
+#   * CSvcov: a vcov as returned by clubSandwich's vcovCR function [class c("vcovCR", "clubSandwich")]
+#   * index: the index belonging to a plm object/model
+# return value:
+#   * modified CSvcov (substituted attribute "cluster" with suitable character or NULL)
+trans_clubSandwich_vcov <- function(CSvcov, index) {
+  clustervar <- attr(CSvcov, "cluster")
+  if (!is.null(clustervar)) {
+    if (isTRUE(all.equal(index[[1L]], clustervar))) {
+      attr(CSvcov, "cluster") <- "group"
+      return(CSvcov)
+    }
+    if (isTRUE(all.equal(index[[2L]], clustervar))) {
+      attr(CSvcov, "cluster") <- "time"
+      return(CSvcov)
+    } else {
+      attr(CSvcov, "cluster") <- NULL
+      return(CSvcov)
+    }
+  }
+  warning("no attribute \"cluster\" found in supplied vcov object")
+  return(CSvcov)
+}
+
+
+
 #' Test of Poolability
 #' 
 #' A Chow test for the poolability of the data.
@@ -969,13 +1038,13 @@ pooltest <- function(x,...){
 #' @export
 pooltest.plm <- function(x, z, ...){
   rss <- deviance(x)
-  uss <- sum(unlist(residuals(z))^2)
+  uss <- as.numeric(crossprod(residuals(z)))
   dlr <- df.residual(x)
   dlu <- df.residual(z)
-  df1 <- dlr-dlu
+  df1 <- dlr - dlu
   df2 <- dlu
   stat <- (rss-uss)/uss*df2/df1
-  pval <- pf(stat, df1, df2, lower.tail = FALSE)
+  pval <- pf(stat, df1 = df1, df2 = df2, lower.tail = FALSE)
   parameter <- c(df1 = df1, df2 = df2)
   names(stat) <- "F"
   res <- list(statistic   = stat,
@@ -992,17 +1061,17 @@ pooltest.plm <- function(x, z, ...){
 #' @export
 pooltest.formula <- function(x, data, ...){
   cl <- match.call(expand.dots = TRUE)
-  cl[[1]] <- as.name("plm")
-  names(cl)[[2]] <- "formula"
+  cl[[1L]] <- as.name("plm")
+  names(cl)[[2L]] <- "formula"
   if (is.null(cl$effect)) cl$effect <- "individual"
-  plm.model <- eval(cl,parent.frame())
+  plm.model <- eval(cl, parent.frame())
 
-  cl[[1]] <- as.name("pvcm")
-  names(cl)[[2]] <- "formula"
+  cl[[1L]] <- as.name("pvcm")
+  names(cl)[[2L]] <- "formula"
   if (is.null(cl$effect)) cl$effect <- "individual"
   cl$model <- "within"
-  pvcm.model <- eval(cl,parent.frame())
+  pvcm.model <- eval(cl, parent.frame())
   
-  pooltest(plm.model,pvcm.model)
+  pooltest(plm.model, pvcm.model)
 }
 
