@@ -2,6 +2,7 @@
 
 ## - trace : calculate trace of a matrix (used in ercomp())
 ## - is.constant : check if a numeric vector or columns of a matrix is constant
+## - sumres : gives summary for residuals (used in several print.summary.* methods)
 ## - bdiag : takes matrices as argument and returns the block-diagonal matrix (used in pgmm and plm.list)
 ## - mylm : inner fitting func based on stats::lm with matrix inputs (used in plm.fit)
 ## - my.lm.fit : like the barebone stats::lm.fit but with some extra information (e.g., SEs, sigma) used in purtest
@@ -13,17 +14,20 @@
 ## - myvar : calculates variance with NA removal, checks if input is constant (also for factor and character)
 ## - pvar : checks if input varies in individual / time dimension
 ## - make.dummies : create a contrast-coded dummy matrix from a factor
+## - describe : extract characteristics of plm and other model objects
 
-trace <- function(x) sum(diag(x))
+trace <- function(x, names = FALSE) sum(diag(x, names = names)) # opposite default for names for speed
 
 is.constant <- function(x) (max(x) - min(x)) < sqrt(.Machine$double.eps)
 
+sumres <- function(x){
+  sr <- summary(unclass(resid(x)))
+  if(abs(sr["Mean"]) < 1e-10) sr[c(1:3, 5:6)] else sr
+}
+
 bdiag <- function(...){
   ## non-exported
-  if (nargs() == 1L)
-    x <- as.list(...)
-  else
-    x <- list(...)
+  x <- if(nargs() == 1L) as.list(...) else list(...)
   n <- length(x)
   if(n == 0L) return(NULL)
   x <- lapply(x, function(y) if(length(y)) as.matrix(y) else
@@ -41,12 +45,12 @@ bdiag <- function(...){
   ind[2,   ] <- rcum
   ind[3, -1] <- ccum[-n]
   ind[4,   ] <- ccum
-  imat <- array(1:(rsum * csum), c(rsum, csum))
+  imat <- array(seq_len(rsum * csum), c(rsum, csum))
   iuse <- apply(ind, 2, function(y, imat) imat[(y[1L]+1):y[2L],
                                                (y[3L]+1):y[4L]], imat = imat)
   iuse <- as.vector(unlist(iuse))
   out[iuse] <- unlist(x)
-  return(out)
+  out
 }
 
 # mylm is used in plm.fit()
@@ -88,7 +92,7 @@ my.lm.fit <- function(X, y, dfcor = TRUE, ...){
   Qr <- reg$qr
   n <- NROW(Qr$qr)
   rdf <- n - p
-  p1 <- 1L:p
+  p1 <- seq_len(p)
   r <- reg$residuals
   rss <- as.numeric(crossprod(r))
   resvar <- if (dfcor) rss/rdf else rss/n
@@ -110,8 +114,8 @@ twosls <- function(y, X, W, intercept = FALSE, lm.type = "lm"){
 
   # As NA/NaN/(+/-)Inf-freeness needs to be guaranteed when functions call
   # twosls(), so can use lm.fit to calc. Xhat.
-  Xhat <- lm.fit(cbind(1, W), X)$fitted.values
-  # old: Xhat <- lm(X ~ W)$fitted.values
+   Xhat <- lm.fit(W, X)$fitted.values
+
   
   if(!is.matrix(Xhat)) {
     # ensure Xhat is a matrix
@@ -157,8 +161,10 @@ data.name <- function(x) {
 #'     `plm` or `panelmodel`),
 #' @param rhs an integer (length > 1 is possible), indicating the parts of right
 #'      hand sides of the formula to be evaluated for the presence of an
-#'      intercept or NULL for all parts of the right hand side
-#'      (relevant for the `Formula` and the `plm` methods)
+#'      intercept or `NULL` for all parts of the right hand side
+#'      (relevant for the `Formula` and the `plm` methods),
+#' @param data default is `NULL` and only needs to be changes to a data set if
+#'             the formula contains a dot (`.`) to allow evaluation of the dot,
 #' @param \dots further arguments.
 #'
 #' @return a logical
@@ -169,27 +175,27 @@ has.intercept <- function(object, ...) {
 
 #' @rdname has.intercept
 #' @export
-has.intercept.default <- function(object, ...) {
-  has.intercept(formula(object), ...)
+has.intercept.default <- function(object, data = NULL, ...) {
+  has.intercept(formula(object), data = data, ...)
 }
 
 #' @rdname has.intercept
 #' @export
-has.intercept.formula <- function(object, ...) {
-  attr(terms(object), "intercept") == 1L
+has.intercept.formula <- function(object, data = NULL, ...) {
+  attr(terms(object, data = data, ...), "intercept") == 1L
 }
 
 #' @rdname has.intercept
 #' @export
-has.intercept.Formula <- function(object, rhs = NULL, ...) {
+has.intercept.Formula <- function(object, rhs = NULL, data = NULL, ...) {
   ## NOTE: returns a logical vector of the necessary length
   ## (which might be > 1)
-  if (is.null(rhs)) rhs <- 1:length(attr(object, "rhs"))
+  if (is.null(rhs)) rhs <- seq_along(attr(object, "rhs"))
   res <- sapply(rhs, function(x) {
     aform <- formula(object, lhs = 0, rhs = x)
     # expand the dot if any in all the parts except the first
-    if (x > 1L) aform <- update(formula(object, lhs = 0, rhs = 1), aform)
-    has.intercept(aform)
+    if (x > 1L) aform <- update(formula(object, lhs = 0, rhs = 1), aform, ...)
+    has.intercept(aform, data = data)
   })
   return(res)
 }
@@ -375,9 +381,8 @@ punbalancedness <- function(x, ...) {
   UseMethod("punbalancedness")
 }
 
-
+#' @export
 punbalancedness.default <- function(x, ...) {
-
   ii <- index(x)
   if(!is.index(ii)) stop("no valid index found for input object 'x'")
   
@@ -401,9 +406,9 @@ punbalancedness.default <- function(x, ...) {
       tss <- ii[[2L]]
       gps <- ii[[3L]]
       Tis <- unique(data.frame(tss, gps))
-      Tis <- table(Tis$gps)               # no of max time periods per group
+      Tis <- collapse::qtable(Tis$gps)    # no of max time periods per group
       Nis <- unique(data.frame(ids, gps))
-      Nis <- table(Nis$gps)               # no of individuals per group
+      Nis <- collapse::qtable(Nis$gps)    # no of individuals per group
       M <- length(unique(gps))            # no of unique groups
       Nbar <- sum(Nis)/M
       Tbar <- sum(Tis)/M
@@ -414,7 +419,7 @@ punbalancedness.default <- function(x, ...) {
       result <- (c(c1 = c1, c2 = c2, c3 = c3))
     } else stop(paste0("unsupported number of dimensions: ", ncol(ii)))
   }
-  return(result)
+  result
 }
 
 #' @rdname punbalancedness
@@ -449,7 +454,7 @@ myvar <- function(x){
     if(n == 1L) z <- 0
   } else {
     z <- if(!(is.factor(x) || is.character(x))) var(x)
-         else !all(duplicated(x)[-1L])
+         else !all(collapse::fduplicated(x, all = FALSE)[-1L])
   }
   z
 }
@@ -521,6 +526,7 @@ pvar <- function(x, ...){
   UseMethod("pvar")
 }
 
+#' @export
 pvar.default <- function(x, id, time, ...){
   name.var <- names(x)
   len <- length(x)
@@ -621,6 +627,12 @@ pvar.pseries <- function(x, ...){
   # use drop.index = TRUE so that the index columns' 
   # variations are not evaluated:
   pdfx <- pseries2pdataframe(x, drop.index = TRUE)
+
+  # give the variable in pdata.frame the original name, so it prints nicely
+  substx <- substitute(x)
+  len <- length(substx)
+  names(pdfx) <- paste(substx)[len]
+
   pvar.pdata.frame(pdfx)
 }
 
@@ -654,12 +666,13 @@ print.pvar <- function(x, ...){
 
 #' Create a Dummy Matrix
 #' 
-#' Contrast-coded dummy matrix created from a factor
+#' Contrast-coded dummy matrix (treatment coding) created from a factor
 #' 
-#' This function creates a matrix of dummies from the levels of a factor.
-#' In model estimations, it is usually preferable to not create the dummy matrix 
-#' prior to estimation but to simply specify a factor in the formula and let the
-#' estimation function handle the creation of the dummies.
+#' This function creates a matrix of dummies from the levels of a factor in 
+#' treatment coding. In model estimations, it is usually preferable to not 
+#' create the dummy matrix prior to estimation but to simply specify a factor 
+#' in the formula and let the estimation function handle the creation of the 
+#' dummies.
 #' 
 #' This function is merely a convenience wrapper around `stats::contr.treatment`
 #' to ease the dummy matrix creation process shall the dummy matrix be explicitly
@@ -686,7 +699,8 @@ print.pvar <- function(x, ...){
 #'            specify the column which is used to derive the dummies from,
 #' @param \dots further arguments.
 #'
-#' @return For the default method, a matrix containing the contrast-coded dummies, 
+#' @return For the default method, a matrix containing the contrast-coded 
+#'         dummies (treatment coding), 
 #'         dimensions are n x n where `n = length(levels(x))` if argument  
 #'        `base.add = TRUE` or `n = length(levels(x)-1)` if `base.add = FALSE`;
 #'         for the data frame and pdata.frame method, a data frame or pdata.frame,
@@ -735,14 +749,14 @@ make.dummies <- function(x, ...){
 make.dummies.default <- function(x, base = 1L, base.add = TRUE, ...) {
   
   stopifnot(is.numeric(base) || is.character(base))
-  if(is.numeric(base)) if(round(base) != base) stop("Argument 'ref' specified as numeric but is not integer")
+  if(is.numeric(base)) if(round(base) != base) stop("Argument 'base' specified as numeric but is not integer")
   if(!is.factor(x)) x <- factor(x)
   
   lvl <- levels(x)
   
   if(is.character(base)) {
     pos <- match(base, lvl)
-    if(is.na(pos)) stop(paste0("argument 'ref' specified as character but value \"", 
+    if(is.na(pos)) stop(paste0("argument 'base' specified as character but value \"", 
                                base, "\", is not in levels(x)"))
     base <- pos
   }
@@ -786,4 +800,34 @@ make.dummies.pdata.frame <- function(x, col, base = 1L, base.add = TRUE, ...) {
 }
 
 
+gettvalue <- function(x, coefname) {
+  ## non-exported
+  ## helper function to extract one or more t value(s)
+  ## (coef/s.e.) for a coefficient from model object useful if one wants
+  ## to avoid the computation of a whole lot of values with summary()
+  
+  # x: model object (usually class plm or lm) coefname: character
+  # indicating name(s) of coefficient(s) for which the t value(s) is
+  # (are) requested
+  # return value: named numeric vector of length == length(coefname)
+  # with requested t value(s)
+  beta <- coef(x)[coefname]
+  se <- sqrt(diag(vcov(x))[coefname])
+  tvalue <- beta / se
+  return(tvalue)
+}
 
+# describe function: extract characteristics of plm and other model objects
+describe <- function(x, what = c("model", "effect", "random.method",
+                                 "inst.method", "transformation", "ht.method")){
+  what <- match.arg(what)
+  cl <- x$args
+  switch(what,
+         "model"          = if(!is.null(cl$model))          cl$model          else "within",
+         "effect"         = if(!is.null(cl$effect))         cl$effect         else "individual",
+         "random.method"  = if(!is.null(cl$random.method))  cl$random.method  else "swar",
+         "inst.method"    = if(!is.null(cl$inst.method))    cl$inst.method    else "bvk",
+         "transformation" = if(!is.null(cl$transformation)) cl$transformation else "d",
+         "ht.method"      = if(!is.null(cl$ht.method))      cl$ht.method      else "ht"
+  )
+}
